# Simulation code to accompany the paper "Evaluating measurement invariance 
#           in categorical data latent variable models with the EPC-interest"
# This reproduces Table 1 in the paper.
# Author: DL Oberski (daniel.oberski@gmail.com)
# Date: 2015-07-01

# NOTE: 
#   This simulation uses simulates data, calls Latent GOLD 5, then uses awk and sed
#        to process the results. This means you need LG5 as well as awk and sed installed.
#   Since LG is a Windows binary, you also need wine. 
#   This code has been tested on Mac OS and should also work on other *nix systems
#   Under Windows, the code can be adjusted by removing the references to Wine,
#        installing awk and sed, and changing system() below to shell().

library(plyr)


set.seed(56485)

setwd("<set this to simulation directory>") 


get_data <- function(n, J, delta=0) {
  z <- rnorm(n)
  x <- z + rnorm(n)
  
  y <- as.data.frame(matrix(rbinom(n*J, 1, plogis(x)), n))
  if(delta != 0) # generate DIF in V1
    y$V1  <- rbinom(n, 1, plogis(x + delta*z))
  y$z <- z
  
  y  
}

get_ll_val <- function(lst_file){
  s <- readLines(lst_file)
  llval <- NA
  for(ln in 1:length(s)) {
    sspl <- strsplit(s[30], "\t")[[1]]
    if(sspl[1]=="Log-likelihood (LL)") {
      llval <- as.numeric(sspl[2])
      break
    }
  }
  llval
}

sim <- function(nobs, delta) {
  # Simulate one dataset
  dat <- get_data(n=nobs, J=4, delta=delta)
  # Write it to disk
  write.table(dat, "test_data.dat", col.names=TRUE, row.names=FALSE, quote=FALSE)
  
  # Run Latent GOLD on the generated set via wine
  # Change path below to your wine binary
  system('export WINEDEBUG=-all; /Applications/Wine.app/Contents/Resources/bin/wine "~/.wine/drive_c/Program Files/LatentGOLD5.0/lg50.exe" test_model.lgs', wait = TRUE)
  # Read in the results from LG analysis using awk and sed
  system("awk '/EPC\\(other\\)/,/Iteration Detail/' test_model.lst | tail +3 > epc.tab")
  system("sed -i '' -e '$ d' epc.tab")
  # Windows users: See http://stackoverflow.com/questions/7866512/shell-with-grep-sed-awk-in-windows
  # http://sourceforge.net/projects/gnuwin32/files/
  # You need: (g)awk, sed, en (?) tail .
  epc_other <- read.table("epc.tab", header=FALSE, sep="\t", stringsAsFactors = FALSE)
  rownames(epc_other) <- apply(epc_other[,1:3], 1, paste, collapse="")
  
  misspec <- paste("V", 1:4, "<-z", sep="")
  
  epc_other_p1 <- epc_other[misspec, "V4"]
  estimates <- scan("test_parameters.txt", what=double(), quiet = TRUE)
  
  c(est_=estimates, epc_other_=epc_other_p1, score=epc_other[misspec, "V13"], 
    loglik=get_ll_val("test_model.lst"))
}

# Simulation conditions
conditions <- expand.grid(nobs=c(250, 500, 1000), delta=c(0, 0.5, 1))
nsim  <- 200 # Number of simulation replications

run_simulation <- FALSE # Set TRUE to actually run the simulation

if(run_simulation) {
  # This will run the entire simulation. Note that you will need:
  #    - Latent GOLD >= 5 with license
  #    - wine installed
  #    - awk and sed installed
  # On Windows, you do not need wine but you probably need to install awk and sed 
  #   from mingw or cygwin. 
  
  system.time(res <- daply(conditions, .variables = 1:2, .fun = function(cond) { 
    cat(sprintf("\nnobs: %d; delta: %1.1f\n", cond[,'nobs'] , cond[,'delta'] ))
  
    t0 <- system.time(res <- ldply(1:nsim, function(isim) sim(cond[,'nobs'] , cond[,'delta'] ), .progress="text"))
    filename <- sprintf("results/res_%d_%1.1f.dat", cond[,'nobs'] , cond[,'delta'] )
    #write.table(res, filename, sep="\t", row.names=FALSE)
    system(sprintf("gzip %s", filename))
    cat(sprintf("Time taken: %1.3fs.\n", t0['elapsed']))
  
    cbind(median_ests = apply(res, 2, median, na.rm=TRUE), 
      mean_ests=apply(res, 2, mean, na.rm=TRUE), 
      sd_ests=apply(res, 2, sd, na.rm=TRUE))
  }))
}
else { # Simply retrieve the saved simulation results
  # Read the saved simulation results array res from disk
  load(file="results_simulation_5648595823.rdata")
}


# Create the table
library(xtable)

res[,,'score1','mean_ests']

round(aperm(res[,,c("est_1","epc_other_1"),'mean_ests'], c(3,2,1)), 3)


